@@ -1,54 +1,84 @@
+//! This module contains all the Client-Specific logic
+
 use crate::{
     connections::{Connections, Destination},
-    message::{Message, MessageHeader, MessageType},
+    message::Message,
+    metrics,
+    metrics::Metrics,
     streams::mpsc,
 };
 
+#[cfg(test)]
+pub(crate) mod mocks;
+
 mod traits;
 pub use traits::*;
 
-/// Contains all the Streams/Queues used by the client
-pub mod queues;
+mod queues;
+pub use queues::Sender as QueueSender;
 
 use rand::RngCore;
-use std::future::Future;
+use std::sync::Arc;
 
-use log::info;
+use self::connections::establish_connection::EstablishConnectionError;
 
 mod connections;
+mod heartbeat;
+
+#[derive(Debug)]
+pub(crate) enum ConnectError {
+    Establish(EstablishConnectionError),
+}
+
+impl From<EstablishConnectionError> for ConnectError {
+    fn from(other: EstablishConnectionError) -> Self {
+        Self::Establish(other)
+    }
+}
 
 /// The Client instance in general that is responsible for handling
 /// all the interactions with the Server
-pub struct Client {
+pub struct Client<M> {
     server_destination: Destination,
     external_port: u16,
     key: Vec<u8>,
+    metrics: Arc<M>,
 }
 
-impl Client {
-    /// Creates the raw Client instance that is configured to
+impl Client<metrics::Empty> {
+    /// Creates a new Client instance that is configured to
     /// connect to the given Server Destination and authenticate
-    /// using the provided Key
+    /// using the provided Key.
+    ///
+    /// This uses an empty Metrics-Collector meaning that no metrics
+    /// will be collected
     pub fn new(server: Destination, external_port: u16, key: Vec<u8>) -> Self {
         Self {
             server_destination: server,
             external_port,
             key,
+            metrics: Arc::new(metrics::Empty::new()),
         }
     }
+}
 
-    async fn heartbeat_loop(
-        send_queue: tokio::sync::mpsc::UnboundedSender<Message>,
-        wait_time: std::time::Duration,
-    ) {
-        loop {
-            let msg_header = MessageHeader::new(0, MessageType::Heartbeat, 0);
-            let msg = Message::new(msg_header, Vec::new());
-            if let Err(e) = send_queue.send(msg) {
-                log::error!("Sending Heartbeat: {}", e);
-                return;
-            };
-            tokio::time::sleep(wait_time).await;
+impl<M> Client<M>
+where
+    M: Metrics + Send + Sync + 'static,
+{
+    /// Behaves the same as the `new` implementation with the difference being
+    /// that you can specify and provide your own Metrics-Collector.
+    pub fn new_metrics(
+        server: Destination,
+        external_port: u16,
+        key: Vec<u8>,
+        metrics_collector: M,
+    ) -> Self {
+        Self {
+            server_destination: server,
+            external_port,
+            key,
+            metrics: Arc::new(metrics_collector),
         }
     }
 
@@ -63,46 +93,34 @@ impl Client {
         ))?;
 
         match max_time {
-            Some(max) => {
-                if calced_result > max {
-                    Some(max)
-                } else {
-                    Some(calced_result)
-                }
-            }
-            None => Some(calced_result),
+            Some(max) if calced_result > max => Some(max),
+            _ => Some(calced_result),
         }
     }
 
     /// Establishes and then also runs a new Connection
-    async fn start_con<F, Fut, T>(
-        &self,
-        start_handler: &F,
-        start_handler_data: &Option<T>,
-    ) -> Result<(), ()>
+    ///
+    /// # Behaviour
+    /// This starts 2 more tasks needed for the Client to work
+    /// properly and then blocks on the 3. function.
+    /// Therefore this function should only return once the
+    /// Connection is being terminated
+    async fn start_con<H>(&self, handler: Arc<H>) -> Result<(), ConnectError>
     where
-        F: Fn(u32, mpsc::StreamReader<Message>, queues::Sender, Option<T>) -> Fut,
-        Fut: Future + Send + 'static,
-        Fut::Output: Send,
-        T: Sized + Send + Clone,
+        H: Handler + Send + Sync + 'static,
     {
         info!(
             "Conneting to server: {}",
             self.server_destination.get_full_address()
         );
 
-        let (read_con, write_con) = match connections::establish_connection::establish_connection(
+        let (read_con, write_con) = connections::establish_connection::establish_connection(
             self.server_destination.get_full_address(),
             &self.key,
             self.external_port,
         )
-        .await
-        {
-            Some(c) => c.into_split(),
-            None => {
-                return Err(());
-            }
-        };
+        .await?
+        .into_split();
 
         info!("Connected to server");
 
@@ -111,14 +129,18 @@ impl Client {
 
         // The Heartbeat loop used to keep the Connection open and verify that it
         // is still working
-        tokio::task::spawn(Self::heartbeat_loop(
+        tokio::task::spawn(heartbeat::keep_alive(
             queue_tx.clone(),
             std::time::Duration::from_secs(15),
         ));
 
         // Runs the Sender in the Background
         // This task is responsible for sending out all the Queued up Messages
-        tokio::task::spawn(connections::tx::sender(write_con, queue_rx));
+        tokio::task::spawn(connections::tx::sender(
+            write_con,
+            queue_rx,
+            self.metrics.clone(),
+        ));
 
         // This task is responsible for receiving all the Messages by the Server
         // and adds them to the fitting Queue
@@ -126,8 +148,8 @@ impl Client {
             read_con,
             queue_tx.clone(),
             outgoing,
-            start_handler,
-            start_handler_data,
+            handler,
+            self.metrics.clone(),
         )
         .await;
 
@@ -148,23 +170,22 @@ impl Client {
     /// The `start_handler` is only ever called once for every new connection in a
     /// seperate tokio::Task
     /// All Messages the Handler receives only Data or EOF Messages
-    pub async fn start<F, Fut, T>(self, start_handler: F, start_handler_data: Option<T>) -> !
+    pub async fn start<H>(self, handler: Arc<H>) -> !
     where
-        F: Fn(u32, mpsc::StreamReader<Message>, queues::Sender, Option<T>) -> Fut,
-        Fut: Future + Send + 'static,
-        Fut::Output: Send,
-        T: Sized + Send + Clone,
+        H: Handler + Send + Sync + 'static,
     {
         info!("Starting...");
 
         let mut attempts = 0;
 
         loop {
-            match self.start_con(&start_handler, &start_handler_data).await {
+            match self.start_con(handler.clone()).await {
                 Ok(_) => {
                     attempts = 0;
                 }
-                Err(_) => {
+                Err(e) => {
+                    error!("Connecting: {:?}", e);
+
                     attempts += 1;
                     if let Some(wait_time) = Self::exponential_backoff(
                         attempts,
