@@ -1,6 +1,10 @@
+use std::sync::Arc;
+
 use crate::{
+    client::QueueSender,
     message::Message,
     streams::{error::RecvError, mpsc},
+    Details,
 };
 
 use async_trait::async_trait;
@@ -24,7 +28,7 @@ pub trait Receiver {
 #[async_trait]
 pub trait Sender {
     /// Sends a single Message over the Connection
-    async fn send_msg(&self, data: Vec<u8>, lenth: u64) -> bool;
+    async fn send_msg(&self, data: Vec<u8>, length: u64) -> bool;
 }
 
 #[async_trait]
@@ -40,3 +44,19 @@ impl Sender for queues::Sender {
         self.send(data, length).await
     }
 }
+
+/// This defines a single Handler that will receive every new Connection
+/// that is established
+#[async_trait]
+pub trait Handler {
+    /// This method is called every time a new Connection is received and
+    /// should therefore handle all the initial stuff for dealing with
+    /// the new Connection
+    async fn new_con(
+        self: Arc<Self>,
+        id: u32,
+        details: Details,
+        rx: mpsc::StreamReader<Message>,
+        tx: QueueSender,
+    );
+}
